import { createContext, useContext } from 'react';
import { useUnit, useStoreMap } from 'effector-react';
import { buildPath } from 'atomic-router';
import { jsx, Fragment } from 'react/jsx-runtime';
import { combine, withFactory } from 'effector';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function r(e){var t,f,n="";if("string"==typeof e||"number"==typeof e)n+=e;else if("object"==typeof e)if(Array.isArray(e))for(t=0;t<e.length;t++)e[t]&&(f=r(e[t]))&&(n&&(n+=" "),n+=f);else for(t in e)e[t]&&(n&&(n+=" "),n+=t);return n}function clsx(){for(var e,t,f=0,n="";f<arguments.length;)(e=arguments[f++])&&(t=r(e))&&(n&&(n+=" "),n+=t);return n}

var RouterContext = /*#__PURE__*/createContext(null);
function RouterProvider(_ref) {
  var router = _ref.router,
      children = _ref.children;
  return /*#__PURE__*/jsx(RouterContext.Provider, {
    value: router,
    children: children
  });
}
function useRouter() {
  return useContext(RouterContext);
}

var _excluded = ["to", "params", "query", "className", "activeClassName", "inactiveClassName"],
    _excluded2 = ["className"],
    _excluded3 = ["to", "params", "query", "className", "activeClassName", "inactiveClassName", "onClick"];
function Link(_ref) {
  var to = _ref.to,
      params = _ref.params,
      query = _ref.query,
      className = _ref.className,
      _ref$activeClassName = _ref.activeClassName,
      activeClassName = _ref$activeClassName === void 0 ? "active" : _ref$activeClassName,
      inactiveClassName = _ref.inactiveClassName,
      props = _objectWithoutPropertiesLoose(_ref, _excluded);

  if (typeof to === "string") {
    return /*#__PURE__*/jsx(NormalLink, _objectSpread2({
      href: to,
      className: className
    }, props));
  }

  return /*#__PURE__*/jsx(RouteLink, _objectSpread2({
    to: to,
    params: params,
    query: query,
    className: className,
    activeClassName: activeClassName,
    inactiveClassName: inactiveClassName
  }, props));
}

function NormalLink(_ref2) {
  var className = _ref2.className,
      props = _objectWithoutPropertiesLoose(_ref2, _excluded2);

  return /*#__PURE__*/jsx("a", _objectSpread2({
    className: clsx(className)
  }, props));
}

function RouteLink(_ref3) {
  var to = _ref3.to,
      params = _ref3.params,
      query = _ref3.query,
      className = _ref3.className,
      _ref3$activeClassName = _ref3.activeClassName,
      activeClassName = _ref3$activeClassName === void 0 ? "active" : _ref3$activeClassName,
      inactiveClassName = _ref3.inactiveClassName,
      _onClick = _ref3.onClick,
      props = _objectWithoutPropertiesLoose(_ref3, _excluded3);

  var router = useRouter();
  var routeObj = router.routes.find(function (routeObj) {
    return routeObj.route === to;
  });

  if (!routeObj) {
    throw new Error("[RouteLink] Route not found");
  }

  var _useUnit = useUnit([routeObj.route.$isOpened, to.navigate]),
      isOpened = _useUnit[0],
      navigate = _useUnit[1];

  var href = buildPath({
    pathCreator: routeObj.path,
    params: params || {},
    query: query || {}
  });
  return /*#__PURE__*/jsx("a", _objectSpread2(_objectSpread2({
    href: href,
    className: clsx(className, isOpened ? activeClassName : inactiveClassName),
    onClick: function onClick(evt) {
      evt.preventDefault();
      navigate({
        params: params || {},
        query: query || {}
      });

      if (_onClick) {
        _onClick(evt);
      }
    }
  }, props), {}, {
    children: props.children
  }));
}

function Route(_ref) {
  var route = _ref.route,
      Component = _ref.view;
  var router = useRouter();
  /* eslint-disable */

  var isOpened = useStoreMap({
    store: router.$activeRoutes,
    keys: [route],
    fn: function fn(activeRoutes, _ref2) {
      var route = _ref2[0];
      return Array.isArray(route) ? route.some(function (route) {
        return activeRoutes.includes(route);
      }) : activeRoutes.includes(route);
    }
  });

  if (isOpened) {
    return /*#__PURE__*/jsx(Component, {});
  }

  return null;
}

var createRouteView = function createRouteView(route, View) {
  var $isOpened = Array.isArray(route) ? combine({
    and: [combine({
      and: [route.map(function (r) {
        return r.$isOpened;
      })],
      or: {
        name: "and",
        sid: "nc98s7"
      }
    }), function (isOpened) {
      return isOpened.includes(true);
    }],
    or: {
      name: "$isOpened",
      sid: "6szwru"
    }
  }) : route.$isOpened;

  function RouteView(props) {
    var isOpened = useUnit($isOpened);

    if (isOpened) {
      return /*#__PURE__*/jsx(View, _objectSpread2({}, props));
    }

    return null;
  }

  return RouteView;
};

var createRoutesView = function createRoutesView(config) {
  var views = config.routes.map(function (_ref) {
    var route = _ref.route,
        view = _ref.view;
    return withFactory({
      sid: "xem4b6",
      fn: function fn() {
        return createRouteView(route, view);
      },
      name: "none",
      method: "createRouteView"
    });
  });
  var $isSomeOpened = combine({
    and: [].concat(config.routes.map(function (_ref2) {
      var route = _ref2.route;
      return route;
    }).flat().map(function (route) {
      return route.$isOpened;
    }), [// @ts-expect-error
    function () {
      for (var _len = arguments.length, isOpened = new Array(_len), _key = 0; _key < _len; _key++) {
        isOpened[_key] = arguments[_key];
      }

      return isOpened.some(Boolean);
    }]),
    or: {
      name: "$isSomeOpened",
      sid: "-lph8u4"
    }
  });
  var NotFound = config.notFound;
  return function () {
    var isSomeOpened = useUnit($isSomeOpened);

    if (!isSomeOpened && NotFound) {
      return /*#__PURE__*/jsx(NotFound, {});
    }

    return /*#__PURE__*/jsx(Fragment, {
      children: views.map(function (View, idx) {
        return /*#__PURE__*/jsx(View, {}, idx);
      })
    });
  };
};

export { Link, Route, RouterContext, RouterProvider, createRouteView, createRoutesView, useRouter };
//# sourceMappingURL=atomic-router.esm.js.map
